"""`texas`
"""

#
# Copyright (c) 2013-2018 by Pawel Tomulik <ptomulik@meil.pw.edu.pl>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE

__docformat__ = "restructuredText"

from .about import __version__

import SCons.Tool.dvipdf
import SCons.Tool.latex
import SCons.Tool.pdflatex
import SCons.Tool.pdftex
import SCons.Tool.tex

from . import TeXASDist
from . import TeXASDoc

try:
    import site_tools.dvipdfm as dvipdfm
except ImportError:
    import sconstool.dvipdfm as dvipdfm

try:
    import site_tools.kpsewhich as kpsewhich
except ImportError:
    import sconstool.kpsewhich as kpsewhich

try:
    import site_tools.archives as archives
except ImportError:
    import sconstool.archives as archives

_archives_generated = False
_dvi_generated = False
_pdf_generated = False
_dvipdfm_generated = False
_kpsewhich_generated = False

def _generate_archives(env):
    global _archives_generated
    if not _archives_generated:
        archives.generate(env)
        env.AddMethod(TeXASDist.Tar, 'TeXASTar')
        env.AddMethod(TeXASDist.TarGz, 'TeXASTarGz')
        env.AddMethod(TeXASDist.TarBz2, 'TeXASTarBz2')
        _archives_generated = True

def _generate_dvi(env):
    global _dvi_generated
    if not _dvi_generated:
        if SCons.Tool.tex.exists(env):
            SCons.Tool.tex.generate(env)
            _dvi_generated = True
        if SCons.Tool.latex.exists(env):
            SCons.Tool.latex.generate(env)
            _dvi_generated = True
        if _dvi_generated:
            env.AddMethod(TeXASDoc.DVI, 'TeXASDVI')

def _generate_pdf(env):
    global _pdf_generated
    if not _pdf_generated:
        if SCons.Tool.dvipdf.exists(env):
            SCons.Tool.dvipdf.generate(env)
            # fix for the problem with out_dir
            env['TEXASDVIPDFCOM'] = 'cd ${TARGET.dir} && ' \
              + '$DVIPDF $DVIPDFFLAGS ${TARGET.rel_path(SOURCE)} ' \
              + '${TARGET.file}'
            _pdf_generated = True
        if SCons.Tool.pdftex.exists(env):
            SCons.Tool.pdftex.generate(env)
            _pdf_generated = True
        if SCons.Tool.pdflatex.exists(env):
            SCons.Tool.pdflatex.generate(env)
            _pdf_generated = True
        if _pdf_generated:
            env.AddMethod(TeXASDoc.PDF, 'TeXASPDF')

def _generate_dvipdfm(env):
    import SCons.Errors
    global _dvipdfm_generated
    if not _dvipdfm_generated:
        dvipdfm.generate(env)
        env.AddMethod(TeXASDoc.DVIPDFM, 'TeXASDVIPDFM')
        # fix for the problem with out_dir
        env['TEXASDVIPDFMCOM'] = 'cd ${TARGET.dir} && ' \
            + '$DVIPDFM $DVIPDFMFLAGS -o ${TARGET.file} ' \
            + '${TARGET.rel_path(SOURCE)}'
        _dvipdfm_generated = True

def _generate_kpsewhich(env):
    import SCons.Errors
    global _kpsewhich_generated
    if not _kpsewhich_generated:
        kpsewhich.generate(env)
        env.AddMethod(TeXASCommon.ImportFromTDS, 'TeXASImport')
        _kpsewhich_generated = True


def generate(env):
    _generate_archives(env)
    _generate_dvi(env)
    _generate_pdf(env)
    _generate_dvipdfm(env)
    _generate_kpsewhich(env)
    env.AddMethod(TeXASDoc.Doc, 'TeXASDoc')
    env.AddMethod(TeXASCommon.RmDup, 'TeXASRmDup')
    env.AddMethod(TeXASCommon.Children, 'TeXASChildren')

def exists(env):
    SCons.Tool.tex.generate_darvin(env)
    return SCons.dvipdf.exists(env) and \
           (SCons.pdftex.exists(env) or SCons.pdflatex.exists(env)) and \
           (SCons.latex.exists(env) or SCons.tex.exists(env) and \
           dvipdfm.exists(env) and kpsewhich.exists(env) and \
           archives.exists(env))

# Local Variables:
# # tab-width:4
# # indent-tabs-mode:nil
# # End:
# vim: set syntax=python expandtab tabstop=4 shiftwidth=4:



"""`TeXASDoc`

Utilities for the compilation of LaTeX papers (articles).
"""

#
# Copyright (c) 2013-2018 by Pawel Tomulik <ptomulik@meil.pw.edu.pl>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE

__docformat__ = "restructuredText"

from . import TeXASCommon

class _Null(object): pass
_null = _Null

_local_keywords = TeXASCommon.keywords  + [
    'deps',
    'builder'
]

def _del_local_keywords(kw):
    global _local_keywords
    return TeXASCommon.del_keys(kw, _local_keywords)

def _builddoc(env, name, source=_null, **kw):
    import SCons.Errors
    try: builder = kw['builder']
    except KeyError: builder = 'DVI'

    if builder == 'DVI' and 'suffix' in kw: del kw['suffix']
    if source is _null: source = name

    alias = TeXASCommon.get_auto_alias(name, 'alias', **kw)
    target = TeXASCommon.get_auto_target(env, name, **kw)

    try: deps = kw['deps']
    except KeyError: deps = None

    kw = _del_local_keywords(kw)

    if builder == 'DVI':
        target = env.DVI(target, source, **kw)
        if deps: env.Depends(target, deps)
    elif builder == 'PDF':
        try: kw['DVIPDFCOM'] = env['TEXASDVIPDFCOM']
        except KeyError: pass
        target = env.PDF(target, source, **kw)
        if deps: env.Depends(target, deps)
    elif builder == 'DVIPDFM':
        try: kw['DVIPDFMCOM'] = env['TEXASDVIPDFMCOM']
        except KeyError: pass
        target = env.DVIPDFM(target, source, **kw)
        if deps: env.Depends(target, deps)
    else:
        raise SCons.Errors.UserError('Unsupported builder: %r' % builder)
    if alias:
        env.Alias(alias, target)
        env.AlwaysBuild(alias)
    return target

def Doc(env, name, source=_null, **kw):
    """Compile document in one of the supported formats

    :Parameters:
        env
            the SCons Environment object
        name
            package name (string)
        source
            source files to be included in the archive
    :Keywords:
        alias
            SCons alias for this target, if not provided the default alias
            ``<name>-dvi`` or ``<name>-<alias_suffix>`` is used,
            otherwise the alias is set to the value of this argument; to not
            create any alias set ``alias=None``,
        alias_suffix
            alias suffix to be used instead of the default ``dvi`` alias suffix,
        builder
            string - name of the builder used to create target, currently
            supported values are ``'DVI'``, ``'PDF'`` and ``'DVIPDFM'``,
        deps
            extra dependencies for target file
        out_dir
            output directory
        suffix
            ignored, the suffix is hard-coded to ``.dvi``
        target
            target file name to use instead of the default auto-generated name,
        version
            used to generate target name,
    :Returns:
        a list of targets created (single target actually)
    """
    import SCons.Errors
    if 'builder' not in kw:
        kw['builder'] = 'DVI'

    if kw['builder'] == 'DVI':
        # Suffixes are fixed for DVI builder
        kw['default_alias_suffix'] = 'dvi'
        kw['default_suffix'] = '.dvi'
    elif kw['builder'] == 'PDF':
        kw['default_alias_suffix'] = 'pdf'
        try: kw['default_suffix'] = kw['PDFSUFFIX']
        except KeyError:
            kw['default_suffix'] = env.subst('$PDFSUFFIX')
            if not kw['default_suffix']:
                kw['default_suffix'] = '.pdf'
    elif kw['builder'] == 'DVIPDFM':
        kw['default_alias_suffix'] = 'pdf'
        try: kw['default_suffix'] = kw['DVIPDFMSUFFIX']
        except KeyError:
            kw['default_suffix'] = env.subst('$DVIPDFMSUFFIX')
            if not kw['default_suffix']:
                kw['default_suffix'] = '.pdf'
    else:
        raise SCons.Errors.UserError('Unsupported builder: %r' % repr(kw['builder']))

    return _builddoc(env, name, source, **kw)

def DVI(env, name, source=_null, **kw):
    """Compile ``*.dvi`` document with SCons DVI builder.

    :Parameters:
        env
            the SCons Environment object
        name
            package name (string)
        source
            source files to be included in the archive

    :Keywords:
        alias
            SCons alias for this target, if not provided the default alias
            ``<name>-dvi`` or ``<name>-<alias_suffix>`` is used,
            otherwise the alias is set to the value of this argument; to not
            create any alias set ``alias=None``,
        alias_suffix
            alias suffix to be used instead of the default ``dvi`` alias suffix,
        deps
            extra dependencies for DVI target,
        out_dir
            output directory
        suffix
            ignored, the suffix is hard-coded to ``.dvi``
        target
            target file name to use instead of the default auto-generated name,
        version
            used to generate target name,

    :Returns:
        a list of targets created (single target actually)
    """
    kw['builder'] = 'DVI'
    return Doc(env, name, source, **kw)

def PDF(env, name, source=_null, **kw):
    """Compile ``*.pdf`` document with SCons PDF builder.

    :Parameters:
        env
            the SCons Environment object
        name
            package name (string)
        source
            source files to be included in the archive

    :Keywords:
        alias
            SCons alias for this target, if not provided the default alias
            ``<name>-dvi`` or ``<name>-<alias_suffix>`` is used,
            otherwise the alias is set to the value of this argument; to not
            create any alias set ``alias=None``,
        alias_suffix
            alias suffix to be used instead of the default ``dvi`` alias suffix,
        deps
            extra dependencies for PDF target,
        out_dir
            output directory
        suffix
            suffix for the target file, by default the ``$PDFSUFFIX``
            construction variable is used, or ``.pdf`` if ``$PDFSUFFIX`` is not
            set
        target
            target file name to use instead of the default auto-generated name,
        version
            used to generate target name,
    :Returns:
        a list of targets created (single target actually)
    """
    kw['builder'] = 'PDF'
    return Doc(env, name, source, **kw)

def DVIPDFM(env, name, source=_null, **kw):
    """Compile ``*.pdf`` document with SCons DVIPDFM builder.

    :Parameters:
        env
            the SCons Environment object
        name
            package name (string)
        source
            source files to be included in the archive

    :Keywords:
        alias
            SCons alias for this target, if not provided the default alias
            ``<name>-dvi`` or ``<name>-<alias_suffix>`` is used,
            otherwise the alias is set to the value of this argument; to not
            create any alias set ``alias=None``,
        alias_suffix
            alias suffix to be used instead of the default ``dvi`` alias suffix,
        deps
            extra dependencies for PDF target,
        out_dir
            output directory
        suffix
            suffix for the target file, by default the ``$DVIPDFMSUFFIX``
            construction variable is used, or ``.pdf`` if ``$DVIPDFMSUFFIX`` is
            not set
        target
            target file name to use instead of the default auto-generated name,
        version
            used to generate target name,

    :Returns:
        a list of targets created (single target actually)
    """
    kw['builder'] = 'DVIPDFM'
    return Doc(env, name, source, **kw)


# Local Variables:
# # tab-width:4
# # indent-tabs-mode:nil
# # End:
# vim: set syntax=python expandtab tabstop=4 shiftwidth=4:



"""`TeXASCommon`

Utilities used commonly by other `texas` modules
"""

#
# Copyright (c) 2013-2018 by Pawel Tomulik <ptomulik@meil.pw.edu.pl>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE

__docformat__ = "restructuredText"

keywords = [
    'alias',
    'alias_suffix',
    'default_alias_suffix',
    'default_suffix',
    'out_dir',
    'suffix',
    'target',
    'version',
]

def RmDup(env, nodes, *args, **kw):
    """Remove duplicates from list ``nodes``, while preserving order"""
    result = []
    seen = set()
    nodes = env.arg2nodes(nodes, *args, **kw)
    for n in nodes:
        if not n in seen:
            result.append(n)
            seen.add(n)
    return result

def Children(env, node):
    node = env.arg2nodes(node, env.fs.Entry)[0]

    children = []
    def select_files(ss):
        import SCons.Node
        for s in ss:
            if isinstance(s, SCons.Node.FS.Dir):
                select_files(s.all_children())
            elif isinstance(s.disambiguate(), SCons.Node.FS.File):
                children.append(s)

    select_files(node.all_children())

    return RmDup(env, children)


def ImportFromTDS(env, source, **kw):
    """Import a file from TeX Directory Structure

    Use ``kpsewhich`` to search for source files within TeX Directory Structure
    (TDS) and copy them to the target directory ``out_dir``.

    Note: in addition to arguments documented here, this method accepts all
    keyword parameters recognized by ``env.KPSShowPath()`` and
    ``env.KPSFindFiles()``. See documentation of ``kpsewhich tool``.

    :Parameters:

        source
            (a list of) file(s) to be imported

    :Keywords:
        out_dir
            where to import files to, defaults to '.'

    :Returns:
        list of nodes created in the target directory.
    """
    import SCons.Util
    import SCons.Script
    import SCons.Errors
    import re
    import platform

    if not SCons.Util.is_Sequence(source): source = [source]
    source = [ env.subst(str(f)) for f in source ]

    try: out_dir = kw['out_dir']
    except KeyError: out_dir = env.Dir('.')

    found = {}
    for f in source:
        suffix = SCons.Util.silent_intern(SCons.Util.splitext(f)[1])
        if suffix:
            kw['path'] = env.KPSShowPath(suffix, **kw)
            # Remove all '.' (CWD) dir entries
            if platform.system() == 'Windows':
                kw['path'] = re.sub(r'(?:^\.[\\/]*;|;\.[\\/]*$)', r'', kw['path'])
                kw['path'] = re.sub(r';\.[\\/]*;',r';', kw['path'])
            else:
                kw['path'] = re.sub(r'(?:^\./*:|:\./*$)', r'', kw['path'])
                kw['path'] = re.sub(r':\./*:',r':', kw['path'])
            # Find the file in TDS
            found[f] = env.KPSFindFiles(f, **kw)
        else:
            raise SCons.Errors.UserError( "Can't import file '%s' " \
                                        + "which has no suffix." % f)
    target = []
    for dname, sname in found.items():
        dst = env.File(dname, out_dir)
        t = env.Command(dst, sname, SCons.Script.Copy('$TARGET', '$SOURCE'))
        target.extend(t)

    return target

def del_keys(dict_,keys):
    for k in keys:
        if k in dict_:
            del dict_[k]
    return dict_

def append_flags(env, key, flags, **kw):
    """Append new flags to ones existing in ``kw[key]`` or ``env[key]``.

    Return the "sum" of flags defined in ``kw[key]`` (or ``env[key]`` if the
    former is not set) and the ``flags``. If ``kw[key]`` is set, then the
    returned value is a "concatenation" of ``kw[key]`` and ``flags``, otherwise
    if ``env[key]`` is given, the function returns "concatenated" ``env[key]``
    and ``flags``. If none of ``kw[key]`` and ``env[key]`` is given, return
    ``flags``. The result is returned as an instance of ``SCons.Util.CLVar``
    object.

    :Parameters:
        env
            the SCons Environment object,
        key
            the keyword/construction variable holding the original flags,
        flags
            the extra flags to be appended to the original ones,
        kw
            keyword arguments, possibly with ``kw[key]`` set.

    :Returns:
        concatenated flags as ``SCons.Util.CLVar`` list.
    """
    import SCons.Util

    try:
        original = kw[key]
    except KeyError:
        try:
            original = env[key]
        except KeyError:
            original = []

    if SCons.Util.is_String(original):
        original = SCons.Util.CLVar([original])
    elif not isinstance(original, SCons.Util.CLVar):
        original = SCons.Util.CLVar(original)

    flags = original + SCons.Util.CLVar(flags)

    return flags

def get_auto_target(env, name, **kw):
    """Return an automatic target for some entity identified by ``name``.

    If the ``target``  keyword argument is set, return its value. Otherwise make
    and return a SCons Node object which points to a file
    ``[<out_dir>]/<name>[-version]<suffix>``. If ``suffix`` is not set, then
    ``default_suffix`` is used instead.

    :Parameters:

        env
            the SCons construction environment,
        name
            package name (string),

    :Keywords:

        suffix
            (optional) suffix for the target file,
        default_suffix
            default suffix for use when the ``suffix`` is not given,
        version
            (optional) package version,
        out_dir
            (optional) output directory for the target file,
        target
            (optional) the fixed target name to be returned instead of
            composing anything

    :Returns:
        target file as the ``SCons.Node.FS.File`` object (unless ``target``
        argument is provided and is not a Node)
    """
    import SCons.Node.FS
    try:
        target = kw['target']
    except KeyError:
        try:
            suffix = kw['suffix']
        except KeyError:
            try:
                suffix = kw['default_suffix']
            except KeyError:
                suffix = ''
        try:
            version = kw['version']
        except KeyError:
            version = None
        if version:
            target = '%s-%s%s' % (name, version, suffix)
        else:
            target = '%s%s' % (name, suffix)
        try:
            out_dir = kw['out_dir']
        except KeyError:
            out_dir = env.Dir('.')
        else:
            if not isinstance(out_dir, SCons.Node.FS.Dir):
                out_dir = env.Dir(out_dir)
        target = env.File(target, out_dir)

    return target

def get_auto_alias(name, alias_kw, **kw):
    """Return an automatic alias for some entity identified by ``name``.

    If the ``alias``  keyword argument is set, return its value. Otherwise
    compose and return an alias as ``<name>[-<alias_suffix>]``. If
    ``alias_suffix`` is not set, use ``default_alias_suffix`` instead.
    If neither of ``alias_suffix`` and ``default_alias_suffix`` is set,
    simply return ``name``.

    :Parameters:

        name
            package name (string),
        alias_kw
            base name of the keyword arguments holding alias information,
    :Keywords:

        alias_suffix
            (optional) suffix for the alias,
        default_alias_suffix
            default alias suffix, used when the ``suffix`` is not given,
        alias
            (optional) the fixed alias name to be returned instead of composing
            anything

    :Returns:
        an alias (string)
    """
    try:
        alias = kw[alias_kw]
    except KeyError:
        try:
            alias_suffix = kw['%s_suffix' % alias_kw]
        except KeyError:
            try:
                alias_suffix = kw['default_%s_suffix' % alias_kw]
            except KeyError:
                alias_suffix = None
        if alias_suffix:
            alias = '%s-%s' % (name, alias_suffix)
        else:
            alias = name
    return alias

def get_strip_dirs(env, **kw):
    """Prepare list of directory nodes to be stripped from the beginning of
    file names.

    :Parameters:
        env
            the SCons Environment object
    :Keywords:
        strip_dirs
            (optional) directory name (string) or sequence of
            directories (Nodes or strings) to be stripped out;
            if string is found, it is interpreted as a directory name
            relative to the current working directory,
    :Returns:
        list of paths (strings), each one relative to the top level source
        directory (denoted by '#' in SCons); if ``strip_dirs`` is not provided,
        an empty list is returned
    """
    import SCons.Util
    try:
        dirs = kw['strip_dirs']
    except KeyError:
        dirs = []

    if SCons.Util.is_String(dirs):
        dirs = [ env.Dir(dirs) ]
    elif SCons.Util.is_Sequence(dirs):
        dirs = [ env.Dir(d) for d in dirs ]
    elif dirs:
        dirs = [ env.Dir('.') ]
    else:
        dirs = []

    dirs2 = dirs[:]
    for d in dirs:
        s = d.srcnode()
        if d != s:
            dirs2.append(s)
    return dirs2

def joinpathre(dirs):
    import re
    import platform
    paths = [ str(d) for d in dirs ]
    # Escape the paths for sed regular expression
    paths = [ re.sub(r'([\.\[\]:\*])', r'\\\1', p) for p in paths ]
    paths = [ re.sub(r'\/*$', r'/*', p) for p in paths ]
    if platform.system() == 'Windows':
        paths = [ re.sub(r'([&<>|\'`,;=()!^])', r'^\1', p) for p in paths ]
        return '\\^|'.join(paths)
    else:
        return '\\|'.join(paths)

def posixsep(node):
    import SCons.Util
    import os
    def ts(n): return '/'.join(n.split(os.path.sep))
    if SCons.Util.is_List(node):
        return [ts(str(n)) for n in node]
    else:
        return ts(str(node))

# Local Variables:
# # tab-width:4
# # indent-tabs-mode:nil
# # End:
# vim: set syntax=python expandtab tabstop=4 shiftwidth=4:



"""`texas.TeXASDist`

Utility functions for packaging and distribution of sources.
"""

#
# Copyright (c) 2013-2018 by Pawel Tomulik <ptomulik@meil.pw.edu.pl>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE

__docformat__ = "restructuredText"

from . import TeXASCommon

_local_keywords = TeXASCommon.keywords + [
    'strip_dirs'
]

def _del_local_keywords(kw):
    global _local_keywords
    return TeXASCommon.del_keys(kw, _local_keywords)

def _tar(env, name, source, **kw):
    """Core of the `Tar()`, `TarGz()` and `TarBz2()`"""
    import SCons.Util
    import platform

    target = TeXASCommon.get_auto_target(env, name, **kw)
    alias = TeXASCommon.get_auto_alias(name, 'alias', **kw)

    # Some paths are stripped from file names in the archive
    dirs2strip = TeXASCommon.get_strip_dirs(env, **kw)

    target = env.TarFile(target, source, TARFILEMAPPINGS=dirs2strip, **kw)

    if alias:
        env.Alias(alias, target)
        env.AlwaysBuild(alias)
    return target

def Tar(env, name, source, **kw):
    """Create tar archive from source(s).

    This function creates TAR archive containing files listed in the ``source``
    argument. The target file name may be fixed with the ``target`` argument.
    Otherwise, the ``out_dir``, ``name``, ``version`` and ``suffix`` are
    used to compose the target file path - the created archive file is
    ``[<out_dir>/]<name>[-<version>].tar`` (``[]`` means optional part and
    ``<>`` stands for argument substitution).  The target file extension might
    be changed with the ``suffix`` argument. The function also auto-generates
    the alias for this target, so user can build the archive with **scons
    <alias>**. To change the alias, provide an alternative with ``alias``
    argument. If you don't wish the alias to be created at all, set
    ``alias=None``.

    Certain leading paths can be stripped out from file names during package
    creation. These paths are provided with the ``strip_dirs`` argument.

    :Parameters:

        env
            the SCons Environment object
        name
            package name (string)
        source
            source files to be included in the archive

    :Keywords:
        alias
            SCons alias for this target, if not provided the default alias
            ``<name>-tar`` or ``<name>-<alias_suffix>`` is used,
            otherwise the alias is set to the value of this argument; to not
            create any alias set ``alias=None``
        alias_suffix
            alias suffix to be used instead of the default ``tar`` alias suffix
        out_dir
            output directory, where to create the archive
        strip_dirs
            directories to strip out from the paths of files in the archive,
            for example if ``source = ["sub1/foo.txt", "sub2/bar.txt",
            "sub3/geez.txt"]`` and ``strip_dirs = ["sub1", "sub3"]``, the
            resulting archive will contain files ``foo.txt``, ``sub2/bar.txt``
            and ``geez.txt``, you may also set ``strip_dirs=True`` to strip
            out the current working directory

        suffix
            suffix for the target file, by default the ``$TARSUFFIX``
            construction variable is used, or ``.tar`` if ``$TARSUFFIX`` is not
            set
        target
            target file name to use instead of the default auto-generated name
        version
            version of the package, used to generate the name of target file
    :Returns:
        a list of targets created (single target actually)
    """

    kw['default_alias_suffix'] = 'tar'
    kw['default_suffix'] = kw.get('TARFILESUFFIX',
                                  env.subst('$TARFILESUFFIX') or '.tar')
    return _tar(env, name, source, **kw)

def TarGz(env, name, source, **kw):
    """Create gzipped tar archive from source(s).

    This function creates gzipped TAR archive containing files listed in the
    ``source`` argument. The target file name may be fixed with the ``target``
    argument.  Otherwise, the ``out_dir``, ``name``, ``version`` and
    ``suffix`` are used to compose the target file path - the created archive
    file is ``[<out_dir>/]<name>[-<version>].tar.gz`` (``[]`` means optional
    part and ``<>`` stands for argument substitution).  The target file
    extension might be changed with the ``suffix`` argument. The function also
    auto-generates the alias for this target, so user can build the archive
    with **scons <alias>**. To change the alias, provide an alternative with
    ``alias`` argument. If you don't wish the alias to be created at all, set
    ``alias=None``.

    Certain leading paths can be stripped out from file names during package
    creation. These paths are provided with the ``strip_dirs`` argument.

    :Parameters:

        env
            the SCons Environment object
        name
            package name (string)
        source
            source files to be included in the archive

    :Keywords:
        alias
            SCons alias for this target, if not provided the default alias
            ``<name>-tgz`` or ``<name>-<alias_suffix>`` is used,
            otherwise the alias is set to the value of this argument; to not
            create any alias set ``alias=None``
        alias_suffix
            alias suffix to be used instead of the default ``tgz`` alias suffix
        out_dir
            output directory, where to create the archive
        strip_dirs
            directories to strip out from the paths of files in the archive,
            for example if ``source = ["sub1/foo.txt", "sub2/bar.txt",
            "sub3/geez.txt"]`` and ``strip_dirs = ["sub1", "sub3"]``, the
            resulting archive will contain files ``foo.txt``, ``sub2/bar.txt``
            and ``geez.txt``, you may also set ``strip_dirs=True`` to strip
            out the current working directory

        suffix
            suffix for the target file, by default ``.tar.gz`` is used
        target
            target file name to use instead of the default auto-generated name
        version
            version of the package, used to generate the name of target file
    :Returns:
        a list of targets created (single target actually)
    """

    kw['default_alias_suffix'] = 'tgz'
    kw['default_suffix'] = '.tar.gz'
    kw['TARFILEMODE'] = 'w:gz'
    return _tar(env, name, source, **kw)

def TarBz2(env, name, source, **kw):
    """Create bzipped tar archive from source(s).

    This function creates bzipped TAR archive containing files listed in the
    ``source`` argument. The target file name may be fixed with the ``target``
    argument.  Otherwise, the ``out_dir``, ``name``, ``version`` and
    ``suffix`` are used to compose the target file path - the created archive
    file is ``[<out_dir>/]<name>[-<version>].tar.bz2`` (``[]`` means optional
    part and ``<>`` stands for argument substitution).  The target file
    extension might be changed with the ``suffix`` argument. The function also
    auto-generates the alias for this target, so user can build the archive
    with **scons <alias>**. To change the alias, provide an alternative with
    ``alias`` argument. If you don't wish the alias to be created at all, set
    ``alias=None``.

    Certain leading paths can be stripped out from file names during package
    creation. These paths are provided with the ``strip_dirs`` argument.

    :Parameters:

        env
            the SCons Environment object
        name
            package name (string)
        source
            source files to be included in the archive

    :Keywords:
        alias
            SCons alias for this target, if not provided the default alias
            ``<name>-tbz2`` or ``<name>-<alias_suffix>`` is used,
            otherwise the alias is set to the value of this argument; to not
            create any alias set ``alias=None``
        alias_suffix
            alias suffix to be used instead of the default ``tbz2`` alias suffix
        out_dir
            output directory, where to create the archive
        strip_dirs
            directories to strip out from the paths of files in the archive,
            for example if ``source = ["sub1/foo.txt", "sub2/bar.txt",
            "sub3/geez.txt"]`` and ``strip_dirs = ["sub1", "sub3"]``, the
            resulting archive will contain files ``foo.txt``, ``sub2/bar.txt``
            and ``geez.txt``, you may also set ``strip_dirs=True`` to strip
            out the current working directory
        suffix
            suffix for the target file, by default ``.tar.bz2`` is used
        target
            target file name to use instead of the default auto-generated name
        version
            version of the package, used to generate the name of target file
    :Returns:
        a list of targets created (single target actually)
    """

    kw['default_alias_suffix'] = 'tbz2'
    kw['default_suffix'] = '.tar.bz2'
    kw['TARFILEMODE'] = 'w:bz2'
    return _tar(env, name, source, **kw)

# Local Variables:
# # tab-width:4
# # indent-tabs-mode:nil
# # End:
# vim: set syntax=python expandtab tabstop=4 shiftwidth=4:
